@@ -65,6 +65,10 @@ GDK_AVAILABLE_IN_ALL GType gdk_frame_clock_phase_get_type (void) G_GNUC_CONST;
 GDK_AVAILABLE_IN_ALL GType gdk_prop_mode_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_PROP_MODE (gdk_prop_mode_get_type ())
 
+/* enumerations from "gdkseat.h" */
+GDK_AVAILABLE_IN_ALL GType gdk_seat_capabilities_get_type (void) G_GNUC_CONST;
+#define GDK_TYPE_SEAT_CAPABILITIES (gdk_seat_capabilities_get_type ())
+
 /* enumerations from "gdktypes.h" */
 GDK_AVAILABLE_IN_ALL GType gdk_byte_order_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_BYTE_ORDER (gdk_byte_order_get_type ())
@@ -82,6 +86,8 @@ GDK_AVAILABLE_IN_ALL GType gdk_event_mask_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_EVENT_MASK (gdk_event_mask_get_type ())
 GDK_AVAILABLE_IN_ALL GType gdk_gl_error_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_GL_ERROR (gdk_gl_error_get_type ())
+GDK_AVAILABLE_IN_ALL GType gdk_window_type_hint_get_type (void) G_GNUC_CONST;
+#define GDK_TYPE_WINDOW_TYPE_HINT (gdk_window_type_hint_get_type ())
 
 /* enumerations from "gdkvisual.h" */
 GDK_AVAILABLE_IN_ALL GType gdk_visual_type_get_type (void) G_GNUC_CONST;
@@ -96,8 +102,6 @@ GDK_AVAILABLE_IN_ALL GType gdk_window_attributes_type_get_type (void) G_GNUC_CON
 #define GDK_TYPE_WINDOW_ATTRIBUTES_TYPE (gdk_window_attributes_type_get_type ())
 GDK_AVAILABLE_IN_ALL GType gdk_window_hints_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_WINDOW_HINTS (gdk_window_hints_get_type ())
-GDK_AVAILABLE_IN_ALL GType gdk_window_type_hint_get_type (void) G_GNUC_CONST;
-#define GDK_TYPE_WINDOW_TYPE_HINT (gdk_window_type_hint_get_type ())
 GDK_AVAILABLE_IN_ALL GType gdk_wm_decoration_get_type (void) G_GNUC_CONST;
 #define GDK_TYPE_WM_DECORATION (gdk_wm_decoration_get_type ())
 GDK_AVAILABLE_IN_ALL GType gdk_wm_function_get_type (void) G_GNUC_CONST;
