@@ -212,8 +212,10 @@ _gdk_wayland_display_add_seat (GdkWaylandDisplay *display_wayland,
   GdkDisplay *gdk_display = GDK_DISPLAY_OBJECT (display_wayland);
   struct wl_seat *seat;
 
+  display_wayland->seat_version = MIN (version, 5);
   seat = wl_registry_bind (display_wayland->wl_registry,
-                           id, &wl_seat_interface, MIN (version, 4));
+                           id, &wl_seat_interface,
+                           display_wayland->seat_version);
   _gdk_wayland_device_manager_add_seat (gdk_display->device_manager,
                                         id, seat);
   _gdk_wayland_display_async_roundtrip (display_wayland);
@@ -312,8 +314,10 @@ gdk_registry_handle_global (void               *data,
     }
   else if (strcmp (interface, "wl_data_device_manager") == 0)
     {
+      display_wayland->data_device_manager_version = MIN (version, 3);
       display_wayland->data_device_manager =
-        wl_registry_bind (display_wayland->wl_registry, id, &wl_data_device_manager_interface, 1);
+        wl_registry_bind (display_wayland->wl_registry, id, &wl_data_device_manager_interface,
+                          display_wayland->data_device_manager_version);
     }
   else if (strcmp (interface, "wl_subcompositor") == 0)
     {
