@@ -182,6 +182,16 @@ _gdk_window_impl_wayland_init (GdkWindowImplWayland *impl)
   impl->initial_fullscreen_monitor = -1;
 }
 
+/* Keep a list of orphaned dialogs (i.e. without parent) */
+static GList *orphan_dialogs;
+
+static void
+_gdk_wayland_screen_add_orphan_dialog (GdkWindow *window)
+{
+  if (!g_list_find (orphan_dialogs, window))
+    orphan_dialogs = g_list_prepend (orphan_dialogs, window);
+}
+
 /*
  * gdk_wayland_window_update_size:
  * @drawable: a #GdkDrawableImplWayland.
@@ -714,18 +724,23 @@ gdk_wayland_window_configure (GdkWindow *window,
 }
 
 static void
-gdk_wayland_window_sync_parent (GdkWindow *window)
+gdk_wayland_window_sync_parent (GdkWindow *window,
+                                GdkWindow *parent)
 {
   GdkWindowImplWayland *impl = GDK_WINDOW_IMPL_WAYLAND (window->impl);
+  GdkWindowImplWayland *impl_parent = NULL;
   struct xdg_surface *parent_surface;
 
   if (!impl->xdg_surface)
     return;
 
   if (impl->transient_for)
-    {
-      GdkWindowImplWayland *impl_parent = GDK_WINDOW_IMPL_WAYLAND (impl->transient_for->impl);
+    impl_parent = GDK_WINDOW_IMPL_WAYLAND (impl->transient_for->impl);
+  else if (parent)
+    impl_parent = GDK_WINDOW_IMPL_WAYLAND (parent->impl);
 
+  if (impl_parent)
+    {
       /* XXX: Is this correct? */
       if (!impl_parent->surface)
         return;
@@ -738,6 +753,31 @@ gdk_wayland_window_sync_parent (GdkWindow *window)
   xdg_surface_set_parent (impl->xdg_surface, parent_surface);
 }
 
+static void
+gdk_wayland_window_update_dialogs (GdkWindow *window)
+{
+  GList *l;
+
+  if (!orphan_dialogs)
+    return;
+
+  for (l = orphan_dialogs; l; l = l->next)
+    {
+      GdkWindow *w = l->data;
+      GdkWindowImplWayland *impl = GDK_WINDOW_IMPL_WAYLAND (w->impl);
+
+      if (w == window)
+	continue;
+      if (impl->hint != GDK_WINDOW_TYPE_HINT_DIALOG)
+        continue;
+      if (impl->transient_for)
+        continue;
+
+      /* Update the parent relationship only for dialogs without transients */
+      gdk_wayland_window_sync_parent (w, window);
+    }
+}
+
 static void
 gdk_wayland_window_sync_title (GdkWindow *window)
 {
@@ -991,6 +1031,9 @@ xdg_surface_configure (void               *data,
   _gdk_set_window_state (window, new_state);
   gdk_wayland_window_sync_margin (window);
   xdg_surface_ack_configure (xdg_surface, serial);
+  if (impl->hint != GDK_WINDOW_TYPE_HINT_DIALOG &&
+      new_state & GDK_WINDOW_STATE_FOCUSED)
+    gdk_wayland_window_update_dialogs (window);
 }
 
 static void
@@ -1032,7 +1075,7 @@ gdk_wayland_window_create_xdg_surface (GdkWindow *window)
   impl->xdg_surface = xdg_shell_get_xdg_surface (display_wayland->xdg_shell, impl->surface);
   xdg_surface_add_listener (impl->xdg_surface, &xdg_surface_listener, window);
 
-  gdk_wayland_window_sync_parent (window);
+  gdk_wayland_window_sync_parent (window, NULL);
   gdk_wayland_window_sync_title (window);
   gdk_wayland_window_sync_margin (window);
 
@@ -1049,6 +1092,9 @@ gdk_wayland_window_create_xdg_surface (GdkWindow *window)
 
   maybe_set_gtk_surface_dbus_properties (window);
   maybe_set_gtk_surface_modal (window);
+
+  if (impl->hint == GDK_WINDOW_TYPE_HINT_DIALOG)
+    _gdk_wayland_screen_add_orphan_dialog (window);
 }
 
 static void
@@ -1130,8 +1176,8 @@ gdk_wayland_window_create_xdg_popup (GdkWindow      *window,
   GdkWaylandDisplay *display = GDK_WAYLAND_DISPLAY (gdk_window_get_display (window));
   GdkWindowImplWayland *impl = GDK_WINDOW_IMPL_WAYLAND (window->impl);
   GdkWindowImplWayland *parent_impl = GDK_WINDOW_IMPL_WAYLAND (parent->impl);
-  GdkDeviceManager *device_manager;
   GdkWaylandDevice *device;
+  GdkSeat *gdk_seat;
   int x, y;
   int parent_x, parent_y;
 
@@ -1141,8 +1187,8 @@ gdk_wayland_window_create_xdg_popup (GdkWindow      *window,
   if (!parent_impl->surface)
     return;
 
-  device_manager = gdk_display_get_device_manager (GDK_DISPLAY (display));
-  device = GDK_WAYLAND_DEVICE (gdk_device_manager_get_client_pointer (device_manager));
+  gdk_seat = gdk_display_get_default_seat (GDK_DISPLAY (display));
+  device = GDK_WAYLAND_DEVICE (gdk_seat_get_pointer (gdk_seat));
 
   gdk_wayland_window_get_fake_root_coords (parent, &parent_x, &parent_y);
 
@@ -1227,36 +1273,27 @@ should_be_mapped (GdkWindow *window)
   return TRUE;
 }
 
-static gboolean
-should_map_as_subsurface (GdkWindow *window)
-{
-  GdkWindowImplWayland *impl = GDK_WINDOW_IMPL_WAYLAND (window->impl);
-
-  if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_SUBSURFACE)
-    return TRUE;
-
-  switch (impl->hint)
-    {
-    case GDK_WINDOW_TYPE_HINT_TOOLTIP:
-      return TRUE;
-
-    case GDK_WINDOW_TYPE_HINT_UTILITY:
-      if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_TEMP)
-        return TRUE;
-      break;
-
-    default:
-      break;
-    }
-
-  return FALSE;
-}
-
 static gboolean
 should_map_as_popup (GdkWindow *window)
 {
   GdkWindowImplWayland *impl = GDK_WINDOW_IMPL_WAYLAND (window->impl);
 
+  /* Ideally, popup would be temp windows with a parent and grab */
+  if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_TEMP)
+    {
+      /* If a temp window has a parent and a grab, we can use a popup */
+      if (impl->transient_for)
+        {
+          if (impl->grab_input_seat)
+            return TRUE;
+        }
+      else
+        g_warning ("Window %p is a temporary window without parent, "
+                   "application will not be able to position it on screen.",
+                   window);
+    }
+
+  /* Yet we need to keep the window type hint tests for compatibility */
   switch (impl->hint)
     {
     case GDK_WINDOW_TYPE_HINT_POPUP_MENU:
@@ -1276,6 +1313,38 @@ should_map_as_popup (GdkWindow *window)
   return FALSE;
 }
 
+static gboolean
+should_map_as_subsurface (GdkWindow *window)
+{
+  GdkWindowImplWayland *impl = GDK_WINDOW_IMPL_WAYLAND (window->impl);
+
+  if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_SUBSURFACE)
+    return TRUE;
+
+  if (GDK_WINDOW_TYPE (window) != GDK_WINDOW_TEMP)
+    return FALSE;
+
+  /* if we want a popup, we do not want a subsurface */
+  if (should_map_as_popup (window))
+    return FALSE;
+
+  if (impl->transient_for)
+    {
+      GdkWindowImplWayland *impl_parent;
+
+      impl_parent = GDK_WINDOW_IMPL_WAYLAND (impl->transient_for->impl);
+      /* subsurface require that the parent is mapped */
+      if (impl_parent->mapped)
+        return TRUE;
+      else
+        g_warning ("Couldn't map window %p as susburface because its parent is not mapped.",
+                   window);
+
+    }
+
+  return FALSE;
+}
+
 /* Get the window that can be used as a parent for a popup, i.e. a xdg_surface
  * or xdg_popup. If the window is not, traverse up the transiency parents until
  * we find one.
@@ -1313,7 +1382,7 @@ gdk_wayland_window_map (GdkWindow *window)
       if (impl->transient_for)
         gdk_wayland_window_create_subsurface (window);
       else
-        g_warning ("Couldn't map as window %p as susburface yet because it doesn't have a parent",
+        g_warning ("Couldn't map window %p as susburface yet because it doesn't have a parent",
                    window);
     }
   else if (should_map_as_popup (window))
@@ -1514,6 +1583,9 @@ gdk_wayland_window_hide_surface (GdkWindow *window)
 
       g_slist_free (impl->outputs);
       impl->outputs = NULL;
+
+      if (impl->hint == GDK_WINDOW_TYPE_HINT_DIALOG && !impl->transient_for)
+        orphan_dialogs = g_list_remove (orphan_dialogs, window);
     }
 
   impl->pending_commit = FALSE;
@@ -1943,19 +2015,48 @@ gdk_wayland_window_set_startup_id (GdkWindow   *window,
 {
 }
 
+static gboolean
+check_transient_for_loop (GdkWindow *window,
+                          GdkWindow *parent)
+{
+  while (parent)
+    {
+      GdkWindowImplWayland *impl = GDK_WINDOW_IMPL_WAYLAND (parent->impl);
+
+      if (impl->transient_for == window)
+        return TRUE;
+      parent = impl->transient_for;
+    }
+  return FALSE;
+}
+
 static void
 gdk_wayland_window_set_transient_for (GdkWindow *window,
                                       GdkWindow *parent)
 {
   GdkWindowImplWayland *impl = GDK_WINDOW_IMPL_WAYLAND (window->impl);
+  GdkWindow *previous_parent;
+
+  if (check_transient_for_loop (window, parent))
+    {
+      g_warning ("Setting %p transient for %p would create a loop", window, parent);
+      return;
+    }
 
   if (impl->subsurface)
     unmap_subsurface (window);
 
+  previous_parent = impl->transient_for;
   impl->transient_for = parent;
 
-  gdk_wayland_window_sync_parent (window);
-
+  if (impl->hint == GDK_WINDOW_TYPE_HINT_DIALOG)
+    {
+      if (!parent)
+        _gdk_wayland_screen_add_orphan_dialog (window);
+      else if (!previous_parent)
+        orphan_dialogs = g_list_remove (orphan_dialogs, window);
+    }
+  gdk_wayland_window_sync_parent (window, NULL);
   if (should_map_as_subsurface (window) &&
       parent && gdk_window_is_visible (window))
     gdk_wayland_window_create_subsurface (window);
