@@ -20,7 +20,7 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
 #include "config.h"
@@ -28,8 +28,6 @@
 #include "gdkprivate-win32.h"
 
 GdkDisplay	 *_gdk_display = NULL;
-GdkScreen	 *_gdk_screen = NULL;
-GdkWindow	 *_gdk_root = NULL;
 
 gint		  _gdk_num_monitors;
 GdkWin32Monitor  *_gdk_monitors = NULL;
