@@ -31,7 +31,7 @@
 #include "gtkprivate.h"
 #include "gtkstock.h"
 #include "gtkiconfactory.h"
-#include "gtkiconfactoryprivate.h"
+#include "deprecated/gtkiconfactoryprivate.h"
 #include "gtkintl.h"
 
 /**
