@@ -55,6 +55,10 @@
  * The #GtkColorChooserWidget is used in the #GtkColorChooserDialog
  * to provide a dialog for selecting colors.
  *
+ * # CSS names
+ *
+ * GtkColorChooserWidget has a single CSS node with name colorchooser.
+ *
  * Since: 3.4
  */
 
@@ -111,14 +115,10 @@ select_swatch (GtkColorChooserWidget *cc,
   gtk_widget_queue_draw (GTK_WIDGET (cc->priv->current));
 
   gtk_color_swatch_get_rgba (swatch, &color);
-
   g_settings_set (cc->priv->settings, "selected-color", "(bdddd)",
                   TRUE, color.red, color.green, color.blue, color.alpha);
 
-  if (gtk_widget_get_visible (GTK_WIDGET (cc->priv->editor)))
-    gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (cc->priv->editor), &color);
-  else
-    g_object_notify (G_OBJECT (cc), "rgba");
+  g_object_notify (G_OBJECT (cc), "rgba");
 }
 
 static void
@@ -703,6 +703,8 @@ gtk_color_chooser_widget_class_init (GtkColorChooserWidgetClass *class)
   g_object_class_install_property (object_class, PROP_SHOW_EDITOR,
       g_param_spec_boolean ("show-editor", P_("Show editor"), P_("Show editor"),
                             FALSE, GTK_PARAM_READWRITE));
+
+  gtk_widget_class_set_css_name (GTK_WIDGET_CLASS (class), "colorchooser");
 }
 
 /* GtkColorChooser implementation {{{1 */
