@@ -122,17 +122,12 @@ static void
 gtk_css_gadget_real_style_changed (GtkCssGadget      *gadget,
                                    GtkCssStyleChange *change)
 {
-  GtkCssGadgetPrivate *priv = gtk_css_gadget_get_instance_private (gadget);
-
-  if (priv->owner)
-    {
-      if (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_SIZE))
-        gtk_widget_queue_resize (priv->owner);
-      else if (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_CLIP))
-        gtk_widget_queue_allocate (priv->owner);
-      else if (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_REDRAW))
-        gtk_widget_queue_draw (priv->owner);
-    }
+  if (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_SIZE))
+    gtk_css_gadget_queue_resize (gadget);
+  else if (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_CLIP))
+    gtk_css_gadget_queue_allocate (gadget);
+  else if (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_REDRAW))
+    gtk_css_gadget_queue_draw (gadget);
 }
 
 static void
@@ -394,6 +389,56 @@ gtk_css_gadget_remove_class (GtkCssGadget *gadget,
   gtk_css_node_remove_class (priv->node, quark);
 }
 
+/**
+ * gtk_css_gadget_set_state:
+ * @gadget: a #GtkCssGadget
+ * @state: The new state
+ *
+ * Sets the state of the gadget's CSS node.
+ */
+void
+gtk_css_gadget_set_state (GtkCssGadget  *gadget,
+                          GtkStateFlags  state)
+{
+  GtkCssGadgetPrivate *priv = gtk_css_gadget_get_instance_private (gadget);
+
+  gtk_css_node_set_state (priv->node, state);
+}
+
+/**
+ * gtk_css_gadget_add_state:
+ * @gadget: a #GtkCssGadget
+ * @state: The state to add
+ *
+ * Adds the given states to the states of gadget's CSS node. Other states
+ * will be kept as they are.
+ */
+void
+gtk_css_gadget_add_state (GtkCssGadget  *gadget,
+                          GtkStateFlags  state)
+{
+  GtkCssGadgetPrivate *priv = gtk_css_gadget_get_instance_private (gadget);
+
+  gtk_css_node_set_state (priv->node, gtk_css_node_get_state (priv->node) | state);
+}
+
+/**
+ * gtk_css_gadget_remove_state:
+ * @gadget: a #GtkCssGadget
+ * @state: The state to remove
+ *
+ * Adds the given states to the states of gadget's CSS node. Other states
+ * will be kept as they are.
+ */
+void
+gtk_css_gadget_remove_state (GtkCssGadget  *gadget,
+                             GtkStateFlags  state)
+{
+  GtkCssGadgetPrivate *priv = gtk_css_gadget_get_instance_private (gadget);
+
+  gtk_css_node_set_state (priv->node, gtk_css_node_get_state (priv->node) & ~state);
+}
+
 static gint
 get_number (GtkCssStyle *style,
             guint        property)
@@ -724,6 +769,55 @@ gtk_css_node_style_changed_for_widget (GtkCssNode  *node,
   _gtk_bitmask_free (changes);
 }
 
+void
+gtk_css_gadget_queue_resize (GtkCssGadget *gadget)
+{
+  g_return_if_fail (GTK_IS_CSS_GADGET (gadget));
+
+  gtk_widget_queue_resize (gtk_css_gadget_get_owner (gadget));
+}
+
+void
+gtk_css_gadget_queue_allocate (GtkCssGadget *gadget)
+{
+  g_return_if_fail (GTK_IS_CSS_GADGET (gadget));
+
+  gtk_widget_queue_allocate (gtk_css_gadget_get_owner (gadget));
+}
+
+void
+gtk_css_gadget_queue_draw (GtkCssGadget *gadget)
+{
+  g_return_if_fail (GTK_IS_CSS_GADGET (gadget));
+  
+  /* XXX: Only invalidate clip here */
+  gtk_widget_queue_draw (gtk_css_gadget_get_owner (gadget));
+}
+
+void
+gtk_css_gadget_get_margin_allocation (GtkCssGadget  *gadget,
+                                      GtkAllocation *allocation,
+                                      int           *baseline)
+{
+  GtkCssGadgetPrivate *priv = gtk_css_gadget_get_instance_private (gadget);
+
+  g_return_if_fail (GTK_IS_CSS_GADGET (gadget));
+
+  if (!gtk_css_gadget_get_visible (gadget))
+    {
+      if (allocation)
+        allocation->x = allocation->y = allocation->width = allocation->height = 0;
+      if (baseline)
+        *baseline = -1;
+      return;
+    }
+
+  if (allocation)
+    *allocation = priv->allocated_size;
+  if (baseline)
+    *baseline = priv->allocated_baseline;
+}
+
 void
 gtk_css_gadget_get_border_allocation (GtkCssGadget  *gadget,
                                       GtkAllocation *allocation,
