@@ -736,8 +736,8 @@ print_pseudoclass_state (const GtkCssSelector *selector,
     "active",
     "hover",
     "selected",
-    "insensitive",
-    "inconsistent",
+    "disabled",
+    "indeterminate",
     "focus",
     "backdrop",
     "dir(ltr)",
@@ -745,7 +745,7 @@ print_pseudoclass_state (const GtkCssSelector *selector,
     "link",
     "visited",
     "checked",
-    "dnd"
+    "drop(active)"
   };
   guint i;
 
@@ -1167,30 +1167,34 @@ parse_selector_pseudo_class (GtkCssParser   *parser,
 {
   static const struct {
     const char    *name;
+    gboolean       deprecated;
     GtkStateFlags  state_flag;
     PositionType   position_type;
     int            position_a;
     int            position_b;
   } pseudo_classes[] = {
-    { "first-child",  0,                           POSITION_FORWARD,  0, 1 },
-    { "last-child",   0,                           POSITION_BACKWARD, 0, 1 },
-    { "only-child",   0,                           POSITION_ONLY,     0, 0 },
-    { "sorted",       0,                           POSITION_SORTED,   0, 0 },
-    { "active",       GTK_STATE_FLAG_ACTIVE, },
-    { "prelight",     GTK_STATE_FLAG_PRELIGHT, },
-    { "hover",        GTK_STATE_FLAG_PRELIGHT, },
-    { "selected",     GTK_STATE_FLAG_SELECTED, },
-    { "insensitive",  GTK_STATE_FLAG_INSENSITIVE, },
-    { "inconsistent", GTK_STATE_FLAG_INCONSISTENT, },
-    { "focused",      GTK_STATE_FLAG_FOCUSED, },
-    { "focus",        GTK_STATE_FLAG_FOCUSED, },
-    { "backdrop",     GTK_STATE_FLAG_BACKDROP, },
-    { "dir(ltr)",     GTK_STATE_FLAG_DIR_LTR, },
-    { "dir(rtl)",     GTK_STATE_FLAG_DIR_RTL, },
-    { "link",         GTK_STATE_FLAG_LINK, },
-    { "visited",      GTK_STATE_FLAG_VISITED, },
-    { "checked",      GTK_STATE_FLAG_CHECKED, },
-    { "dnd"    ,      GTK_STATE_FLAG_DND, }
+    { "first-child",   0, 0,                           POSITION_FORWARD,  0, 1 },
+    { "last-child",    0, 0,                           POSITION_BACKWARD, 0, 1 },
+    { "only-child",    0, 0,                           POSITION_ONLY,     0, 0 },
+    { "sorted",        1, 0,                           POSITION_SORTED,   0, 0 },
+    { "active",        0, GTK_STATE_FLAG_ACTIVE, },
+    { "prelight",      1, GTK_STATE_FLAG_PRELIGHT, },
+    { "hover",         0, GTK_STATE_FLAG_PRELIGHT, },
+    { "selected",      0, GTK_STATE_FLAG_SELECTED, },
+    { "insensitive",   1, GTK_STATE_FLAG_INSENSITIVE, },
+    { "disabled",      0, GTK_STATE_FLAG_INSENSITIVE, },
+    { "inconsistent",  1, GTK_STATE_FLAG_INCONSISTENT, },
+    { "indeterminate", 0, GTK_STATE_FLAG_INCONSISTENT, },
+    { "focused",       1, GTK_STATE_FLAG_FOCUSED, },
+    { "focus",         0, GTK_STATE_FLAG_FOCUSED, },
+    { "backdrop",      0, GTK_STATE_FLAG_BACKDROP, },
+    { "dir(ltr)",      0, GTK_STATE_FLAG_DIR_LTR, },
+    { "dir(rtl)",      0, GTK_STATE_FLAG_DIR_RTL, },
+    { "link",          0, GTK_STATE_FLAG_LINK, },
+    { "visited",       0, GTK_STATE_FLAG_VISITED, },
+    { "checked",       0, GTK_STATE_FLAG_CHECKED, },
+    { "drop(active)",  0, GTK_STATE_FLAG_DROP_ACTIVE, }
+
   };
   guint i;
 
@@ -1209,6 +1213,20 @@ parse_selector_pseudo_class (GtkCssParser   *parser,
                                                       : &GTK_CSS_SELECTOR_PSEUDOCLASS_STATE,
                                                selector);
               selector->state.state = pseudo_classes[i].state_flag;
+              if (pseudo_classes[i].deprecated)
+                {
+                  if (pseudo_classes[i + 1].state_flag == pseudo_classes[i].state_flag)
+                    _gtk_css_parser_error_full (parser,
+                                                GTK_CSS_PROVIDER_ERROR_DEPRECATED,
+                                                "The :%s pseudo-class is deprecated. Use :%s instead.",
+                                                pseudo_classes[i].name,
+                                                pseudo_classes[i + 1].name);
+                  else
+                    _gtk_css_parser_error_full (parser,
+                                                GTK_CSS_PROVIDER_ERROR_DEPRECATED,
+                                                "The :%s pseudo-class is deprecated.",
+                                                pseudo_classes[i].name);
+                }
             }
           else
             {
