@@ -48,7 +48,7 @@ struct _GtkIconHelperPrivate {
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkIconHelper, gtk_icon_helper, GTK_TYPE_CSS_GADGET)
 
-static void
+void
 gtk_icon_helper_invalidate (GtkIconHelper *self)
 {
   if (self->priv->rendered_surface != NULL)
@@ -148,7 +148,8 @@ gtk_icon_helper_style_changed (GtkCssGadget      *gadget,
   if (gtk_css_style_change_affects (change, GTK_CSS_AFFECTS_ICON))
     gtk_icon_helper_invalidate (GTK_ICON_HELPER (gadget));
 
-  GTK_CSS_GADGET_CLASS (gtk_icon_helper_parent_class)->style_changed (gadget, change);
+  if (!GTK_IS_CSS_TRANSIENT_NODE (gtk_css_gadget_get_node (gadget)))
+    GTK_CSS_GADGET_CLASS (gtk_icon_helper_parent_class)->style_changed (gadget, change);
 }
 
 static void
@@ -574,6 +575,9 @@ gtk_icon_helper_ensure_surface (GtkIconHelper *self)
 {
   int scale;
 
+  if (self->priv->rendered_surface)
+    return;
+
   scale = gtk_widget_get_scale_factor (gtk_css_gadget_get_owner (GTK_CSS_GADGET (self)));
 
   self->priv->rendered_surface = gtk_icon_helper_load_surface (self, scale);
