@@ -48,6 +48,7 @@
 #include "gtkorientable.h"
 #include "gtkrender.h"
 #include "gtktypebuiltins.h"
+#include "gtkwidgetprivate.h"
 #include "deprecated/gtkstock.h"
 
 /**
@@ -174,14 +175,6 @@ static void     gtk_info_bar_get_property (GObject        *object,
                                            guint           prop_id,
                                            GValue         *value,
                                            GParamSpec     *pspec);
-static void     gtk_info_bar_get_preferred_width          (GtkWidget *widget,
-                                                           gint      *minimum_width,
-                                                           gint      *natural_width);
-static void     gtk_info_bar_get_preferred_height         (GtkWidget *widget,
-                                                           gint      *minimum_height,
-                                                           gint      *natural_height);
-static gboolean gtk_info_bar_draw                         (GtkWidget      *widget,
-                                                           cairo_t        *cr);
 static void     gtk_info_bar_buildable_interface_init     (GtkBuildableIface *iface);
 static gboolean  gtk_info_bar_buildable_custom_tag_start   (GtkBuildable  *buildable,
                                                             GtkBuilder    *builder,
@@ -245,12 +238,6 @@ gtk_info_bar_get_property (GObject    *object,
     }
 }
 
-static void
-gtk_info_bar_finalize (GObject *object)
-{
-  G_OBJECT_CLASS (gtk_info_bar_parent_class)->finalize (object);
-}
-
 static void
 response_data_free (gpointer data)
 {
@@ -313,79 +300,6 @@ gtk_info_bar_close (GtkInfoBar *info_bar)
                          GTK_RESPONSE_CANCEL);
 }
 
-static void
-get_padding_and_border (GtkWidget *widget,
-                        GtkBorder *border)
-{
-  GtkStyleContext *context;
-  GtkStateFlags state;
-  GtkBorder tmp;
-
-  context = gtk_widget_get_style_context (widget);
-  state = gtk_style_context_get_state (context);
-
-  gtk_style_context_get_padding (context, state, border);
-  gtk_style_context_get_border (context, state, &tmp);
-  border->top += tmp.top;
-  border->right += tmp.right;
-  border->bottom += tmp.bottom;
-  border->left += tmp.left;
-}
-
-static void
-gtk_info_bar_get_preferred_width (GtkWidget *widget,
-                                  gint      *minimum_width,
-                                  gint      *natural_width)
-{
-  GtkBorder border;
-
-  get_padding_and_border (widget, &border);
-
-  GTK_WIDGET_CLASS (gtk_info_bar_parent_class)->get_preferred_width (widget,
-                                                                     minimum_width,
-                                                                     natural_width);
-
-  *minimum_width += border.left + border.right;
-  *natural_width += border.left + border.right;
-}
-
-static void
-gtk_info_bar_get_preferred_height (GtkWidget *widget,
-                                   gint      *minimum_height,
-                                   gint      *natural_height)
-{
-  GtkBorder border;
-
-  get_padding_and_border (widget, &border);
-
-  GTK_WIDGET_CLASS (gtk_info_bar_parent_class)->get_preferred_height (widget,
-                                                                      minimum_height,
-                                                                      natural_height);
-
-  *minimum_height += border.top + border.bottom;
-  *natural_height += border.top + border.bottom;
-}
-
-static gboolean
-gtk_info_bar_draw (GtkWidget *widget,
-                   cairo_t   *cr)
-{
-  GtkStyleContext *context;
-
-  context = gtk_widget_get_style_context (widget);
-
-  gtk_render_background (context, cr, 0, 0,
-                         gtk_widget_get_allocated_width (widget),
-                         gtk_widget_get_allocated_height (widget));
-  gtk_render_frame (context, cr, 0, 0,
-                    gtk_widget_get_allocated_width (widget),
-                    gtk_widget_get_allocated_height (widget));
-
-  GTK_WIDGET_CLASS (gtk_info_bar_parent_class)->draw (widget, cr);
-
-  return FALSE;
-}
-
 static void
 gtk_info_bar_show (GtkWidget *widget)
 {
@@ -428,11 +342,7 @@ gtk_info_bar_class_init (GtkInfoBarClass *klass)
 
   object_class->get_property = gtk_info_bar_get_property;
   object_class->set_property = gtk_info_bar_set_property;
-  object_class->finalize = gtk_info_bar_finalize;
 
-  widget_class->get_preferred_width = gtk_info_bar_get_preferred_width;
-  widget_class->get_preferred_height = gtk_info_bar_get_preferred_height;
-  widget_class->draw = gtk_info_bar_draw;
   widget_class->show = gtk_info_bar_show;
   widget_class->hide = gtk_info_bar_hide;
 
@@ -606,21 +516,22 @@ close_button_clicked_cb (GtkWidget  *button,
 static void
 gtk_info_bar_init (GtkInfoBar *info_bar)
 {
+  GtkInfoBarPrivate *priv;
   GtkWidget *widget = GTK_WIDGET (info_bar);
 
-  info_bar->priv = gtk_info_bar_get_instance_private (info_bar);
+  priv = info_bar->priv = gtk_info_bar_get_instance_private (info_bar);
 
   gtk_widget_set_redraw_on_allocate (widget, TRUE);
 
   /* message-type is a CONSTRUCT property, so we init to a value
    * different from its default to trigger its property setter
    * during construction */
-  info_bar->priv->message_type = GTK_MESSAGE_OTHER;
+  priv->message_type = GTK_MESSAGE_OTHER;
 
-  gtk_widget_init_template (GTK_WIDGET (info_bar));
+  gtk_widget_init_template (widget);
 
-  gtk_widget_set_no_show_all (info_bar->priv->close_button, TRUE);
-  g_signal_connect (info_bar->priv->close_button, "clicked",
+  gtk_widget_set_no_show_all (priv->close_button, TRUE);
+  g_signal_connect (priv->close_button, "clicked",
                     G_CALLBACK (close_button_clicked_cb), info_bar);
 }
 
