@@ -201,7 +201,7 @@ gtk_model_button_update_state (GtkModelButton *button)
 
   state = gtk_widget_get_state_flags (GTK_WIDGET (button));
 
-  gtk_css_node_set_state (gtk_css_gadget_get_node (button->gadget), state);
+  gtk_css_gadget_set_state (button->gadget, state);
 
   if (button->role == GTK_BUTTON_ROLE_CHECK)
     {
@@ -239,7 +239,7 @@ gtk_model_button_update_state (GtkModelButton *button)
     }
 
   gtk_builtin_icon_set_image (GTK_BUILTIN_ICON (button->indicator_gadget), image_type);
-  gtk_css_node_set_state (gtk_css_gadget_get_node (button->indicator_gadget), state);
+  gtk_css_gadget_set_state (button->indicator_gadget, state);
 }
 
 static void
@@ -860,12 +860,6 @@ gtk_model_button_size_allocate (GtkWidget     *widget,
     gadget = GTK_MODEL_BUTTON (widget)->gadget;
 
   gtk_widget_set_allocation (widget, allocation);
-
-  if (gtk_widget_get_realized (widget))
-    gdk_window_move_resize (gtk_button_get_event_window (GTK_BUTTON (widget)),
-                            allocation->x, allocation->y,
-                            allocation->width, allocation->height);
-
   gtk_css_gadget_allocate (gadget,
                            allocation,
                            gtk_widget_get_allocated_baseline (widget),
@@ -959,6 +953,19 @@ gtk_model_button_allocate (GtkCssGadget        *gadget,
     (pango_font_metrics_get_ascent (metrics) + pango_font_metrics_get_descent (metrics));
   pango_font_metrics_unref (metrics);
 
+
+  if (gtk_widget_get_realized (widget))
+    {
+      GtkAllocation border_allocation;
+      gtk_css_gadget_get_border_allocation (gadget, &border_allocation, NULL);
+
+      gdk_window_move_resize (gtk_button_get_event_window (GTK_BUTTON (widget)),
+                              border_allocation.x,
+                              border_allocation.y,
+                              border_allocation.width,
+                              border_allocation.height);
+    }
+
   gtk_container_get_children_clip (GTK_CONTAINER (widget), out_clip);
   gdk_rectangle_union (out_clip, &check_clip, out_clip);
 }
