@@ -511,18 +511,20 @@ render_frame_stroke (cairo_t       *cr,
       GtkRoundedBox padding_box;
 
       padding_box = *border_box;
-      _gtk_rounded_box_path (&padding_box, cr);
       _gtk_rounded_box_shrink (&padding_box,
                                border_width[GTK_CSS_TOP],
                                border_width[GTK_CSS_RIGHT],
                                border_width[GTK_CSS_BOTTOM],
                                border_width[GTK_CSS_LEFT]);
 
-      for (i = 0; i < 4; i++) 
+      for (i = 0; i < 4; i++)
         {
           if (hidden_side & (1 << i))
             continue;
 
+          if (border_width[i] == 0)
+            continue;
+
           cairo_save (cr);
 
           if (i == 0)
@@ -630,7 +632,7 @@ render_border (cairo_t       *cr,
                 else
                   dont_draw |= (1 << j);
                 
-                other_border[i] = border_width[i] / 3;
+                other_border[j] = border_width[j] / 3;
               }
             
             render_frame_fill (cr, border_box, other_border, colors, dont_draw);
