@@ -147,19 +147,9 @@ allocation_changed (GtkWidget *w, GdkRectangle *allocation, GtkInspectorMiscInfo
 
   gtk_widget_get_clip (w, &clip);
 
-  if (clip.width == gtk_widget_get_allocated_width (w) &&
-      clip.height == gtk_widget_get_allocated_height (w))
-    {
-      gtk_widget_hide (sl->priv->clip_area_row);
-    }
-  else
-    {
-      gtk_widget_show (sl->priv->clip_area_row);
-
-      size_label = g_strdup_printf ("%d × %d", clip.width, clip.height);
-      gtk_label_set_label (GTK_LABEL (sl->priv->clip_area), size_label);
-      g_free (size_label);
-    }
+  size_label = g_strdup_printf ("%d × %d", clip.width, clip.height);
+  gtk_label_set_label (GTK_LABEL (sl->priv->clip_area), size_label);
+  g_free (size_label);
 
   class = G_ENUM_CLASS (g_type_class_ref (GTK_TYPE_SIZE_REQUEST_MODE));
   value = g_enum_get_value (class, gtk_widget_get_request_mode (w));
@@ -420,6 +410,7 @@ gtk_inspector_misc_info_set_object (GtkInspectorMiscInfo *sl,
       gtk_widget_show (sl->priv->state_row);
       gtk_widget_show (sl->priv->request_mode_row);
       gtk_widget_show (sl->priv->allocated_size_row);
+      gtk_widget_show (sl->priv->clip_area_row);
       gtk_widget_show (sl->priv->mnemonic_label_row);
       gtk_widget_show (sl->priv->tick_callback_row);
       gtk_widget_show (sl->priv->accessible_role_row);
