@@ -185,6 +185,19 @@ pixelcache_activate (GtkSwitch *sw)
   redraw_everything ();
 }
 
+static void
+widget_resize_activate (GtkSwitch *sw)
+{
+  guint flags = gtk_get_debug_flags ();
+
+  if (gtk_switch_get_active (sw))
+    flags |= GTK_DEBUG_RESIZE;
+  else
+    flags &= ~GTK_DEBUG_RESIZE;
+
+  gtk_set_debug_flags (flags);
+}
+
 static void
 fill_gtk (const gchar *path,
           GHashTable  *t)
@@ -227,11 +240,13 @@ init_theme (GtkInspectorVisual *vis)
   GHashTable *t;
   GHashTableIter iter;
   gchar *theme, *path;
+  GList *list, *l;
 
   t = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   /* Builtin themes */
   g_hash_table_add (t, g_strdup ("Adwaita"));
   g_hash_table_add (t, g_strdup ("HighContrast"));
+  g_hash_table_add (t, g_strdup ("HighContrastInverse"));
   g_hash_table_add (t, g_strdup ("Raleigh"));
 
   path = _gtk_css_provider_get_theme_dir ();
@@ -246,9 +261,18 @@ init_theme (GtkInspectorVisual *vis)
   fill_gtk (path, t);
   g_free (path);
 
+  list = NULL;
   g_hash_table_iter_init (&iter, t);
   while (g_hash_table_iter_next (&iter, (gpointer *)&theme, NULL))
-    gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (vis->priv->theme_combo), theme, theme);
+    list = g_list_insert_sorted (list, theme, (GCompareFunc)strcmp);
+
+  for (l = list; l; l = l->next)
+    {
+      theme = l->data;
+      gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (vis->priv->theme_combo), theme, theme);
+    }
+
+  g_list_free (list);
   g_hash_table_destroy (t);
 
   g_object_bind_property (gtk_settings_get_default (), "gtk-theme-name",
@@ -308,6 +332,7 @@ init_icons (GtkInspectorVisual *vis)
   GHashTable *t;
   GHashTableIter iter;
   gchar *theme, *path;
+  GList *list, *l;
 
   t = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
@@ -319,12 +344,19 @@ init_icons (GtkInspectorVisual *vis)
   fill_icons (path, t);
   g_free (path);
 
+  list = NULL;
   g_hash_table_iter_init (&iter, t);
   while (g_hash_table_iter_next (&iter, (gpointer *)&theme, NULL))
+    list = g_list_insert_sorted (list, theme, (GCompareFunc)strcmp);
+
+  for (l = list; l; l = l->next)
     {
+      theme = l->data;
       gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (vis->priv->icon_combo), theme, theme);
     }
+
   g_hash_table_destroy (t);
+  g_list_free (list);
 
   g_object_bind_property (gtk_settings_get_default (), "gtk-icon-theme-name",
                           vis->priv->icon_combo, "active-id",
@@ -360,6 +392,7 @@ init_cursors (GtkInspectorVisual *vis)
   GHashTable *t;
   GHashTableIter iter;
   gchar *theme, *path;
+  GList *list, *l;
 
   t = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
@@ -371,12 +404,19 @@ init_cursors (GtkInspectorVisual *vis)
   fill_cursors (path, t);
   g_free (path);
 
+  list = NULL;
   g_hash_table_iter_init (&iter, t);
   while (g_hash_table_iter_next (&iter, (gpointer *)&theme, NULL))
+    list = g_list_insert_sorted (list, theme, (GCompareFunc)strcmp);
+
+  for (l = list; l; l = l->next)
     {
+      theme = l->data;
       gtk_combo_box_text_append (GTK_COMBO_BOX_TEXT (vis->priv->cursor_combo), theme, theme);
     }
+
   g_hash_table_destroy (t);
+  g_list_free (list);
 
   g_object_bind_property (gtk_settings_get_default (), "gtk-cursor-theme-name",
                           vis->priv->cursor_combo, "active-id",
@@ -699,6 +739,7 @@ gtk_inspector_visual_class_init (GtkInspectorVisualClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, rendering_mode_changed);
   gtk_widget_class_bind_template_callback (widget_class, baselines_activate);
   gtk_widget_class_bind_template_callback (widget_class, pixelcache_activate);
+  gtk_widget_class_bind_template_callback (widget_class, widget_resize_activate);
   gtk_widget_class_bind_template_callback (widget_class, software_gl_activate);
   gtk_widget_class_bind_template_callback (widget_class, software_surface_activate);
   gtk_widget_class_bind_template_callback (widget_class, texture_rectangle_activate);
